// Copyright (c) 2022 Digital Asset (Switzerland) GmbH and/or its affiliates
//
// Proprietary code. All rights reserved.

pragma solidity 0.8.10;
pragma experimental ABIEncoderV2;

contract Sequencer {

    // the duration to allow the optionally supplied signing timestamp to be from the assigned sequenced time
    // same as signingTolerance set in the DynamicDomainParameters
    uint signingTolerance = 0;
    // who created the sequencer contract and is the only member able to write to the sequencer
    // Time in milliseconds since epoch
    uint256 clock = 0;
    // The ID of the topology manager
    string topologyManager = "";
    // registered members
    string[] members;
    // we store counter values by adding 1 to be able to determine whether the counter has been initialized from the
    // default value of 0.
    mapping (string => uint64) counters;

    // Emitted if a batch of events was successfully sequenced.
    event Deliver(string traceParent, string[] recipients, uint64[] counters, uint256 timestamp, string messageId, bytes batch, uint256 timestampOfSigningKey);
    event DeliverError(string traceParent, string recipient, uint64 counter, uint256 timestamp, string messageId, string reason, string message);
    event NewMember(string traceParent, string member, uint256 timestamp);
    event FatalError(string traceParent, string message);

    function updateClock() private {
        // block.timestamp is an epoch timestamp in seconds
        uint256 newClock = block.timestamp * 1000;
        if (clock >= newClock){
            clock = clock + 1;
        }
        else {
            clock = newClock;
        } 
    }

    /**
        @notice Registers the topology manager. The TPM is responsible for registering members on the sequencer and
        is the only member allowed to register other members. Emits a `FatalError` if the TPM was already set to a
        different value previously.
    */
    function registerIdm(string memory traceParent, string memory idmID) public {
        if (bytes(topologyManager).length == 0){
            topologyManager = idmID;
            if (!isMemberRegistered(idmID)) {
                registerMember(traceParent, idmID);
            }
        }
        else {
            // the solidity == method is "pointer equality"
            bool differs = keccak256(abi.encodePacked((topologyManager))) != keccak256(abi.encodePacked((idmID)));
            if (differs) {
                emit FatalError(traceParent, "Unexpected change in topology manager ID");
            }
        }

    }

    /**
        @notice Set the signing tolerance for the sequencer key. Emits a `FatalError` if the tolerance was already set
        to a different value previously.
    */
    function setSigningTolerance(string memory traceParent,uint signingToleranceMillis) public {
        if(signingTolerance == 0){
            signingTolerance = signingToleranceMillis;
        }
        else if (signingTolerance != signingToleranceMillis) {
            emit FatalError(traceParent, "Unexpected change in signing tolerance");
        }
    }

    function isMemberRegistered(string memory member) private view returns (bool) {
        uint64 counter = counters[member];

        return counter != 0;
    }

    function nextCounter(string memory member) private returns (uint64) {
        uint64 counter = counters[member];
        counters[member] = counter + 1;
        return counter - 1;
    }

    function registerMemberNow(string memory traceParent, string memory newMember) private {
        members.push(newMember);
        counters[newMember] = 1;
        emit NewMember(traceParent, newMember, clock);
    }

    /**
        @notice Register a member starting a sequencer counter for them. Fails if a member was already registered.
        This function should only be called by the topology manager.
    */
    function registerMember(string memory traceParent, string memory newMember) public {
        require(counters[newMember] == 0, "Member is already registered");
        updateClock();
        registerMemberNow(traceParent, newMember);
    }

    /**
        @notice This is the most important function of the sequencer smart contract. Sequence a batch of events assigning them a timestamp.
        @param timestampOfSigningKey should be 0 if unset and an epoch timestamp in milliseconds otherwise.
        @param batch compressed binary batch of events
    */
    function sendAsync(
    string memory traceParent,
        string memory sender,
        string memory messageId,
        uint maxSequencingTime,
        uint timestampOfSigningKey,
        bytes memory batch,
        string[] memory recipients
    ) public {
        updateClock();

        // if the sender doesn't exist at this point just entirely ignore the request as we won't be able to send the
        // DeliverError to anyone
        if (!isMemberRegistered(sender)) {
            return;
        }

        // check we haven't exceeded the max sequencing time
        if (clock > maxSequencingTime) {
            // we can't emit an event using the message-id past the max-sequencing-time.
            // the sender should be monitoring their sequenced events and timeout the
            // send when a higher timestamp is observed.
            return;
        }

        // if the signing key timestamp key is specified, make sure it's within tolerance of the sequencer timestamp we've just generated
        if (timestampOfSigningKey > 0) {
            // lower bound check
            if (signingTolerance > clock) {
                // the duration we allow is greater than the current clock value
                // either we're in the 70s or the signingTolerance is huge
                // but regardless it means whatever the timestampOfSigningKey will be fine for a lower bound
                // (not doing the lower bound calculation also prevents an overflow)
            } else {
                uint256 lowerBoundExcl = clock - signingTolerance;
                if (timestampOfSigningKey <= lowerBoundExcl) {
                    emit DeliverError(traceParent, sender, nextCounter(sender), clock, messageId, 'BatchRefused',
                        'The provided timestamp of signing key is before the range allowed by the configured tolerance around the selected sequenced timestamp');
                    return;
                }
            }

            // upper bound check
            if (timestampOfSigningKey > clock) {
                emit DeliverError(traceParent, sender, nextCounter(sender), clock, messageId, 'BatchRefused',
                    'The provided timestamp of signing key is after the sequenced timestamp');
                return;
            }
        }

        // the solidity == method is "pointer equality"
        // so compare the hashes of the sender and the topologyManager instead of using a direct comparison
        bool fromIdm = keccak256(abi.encodePacked((sender))) == keccak256(abi.encodePacked((topologyManager)));
        // check all recipients exist
        for (uint i = 0; i < recipients.length; i++) {
            if (!isMemberRegistered(recipients[i])) {
                if (fromIdm) {
                    registerMemberNow(traceParent, recipients[i]);
                }
                else {
                    emit DeliverError(traceParent, sender, nextCounter(sender), clock, messageId, 'BatchRefused', 'Recipient is not registered');
                    return;
                }
            }
        }
        uint256 n = recipients.length;
        uint64[] memory newCounters = new uint64[](n);
        for (uint i = 0; i < n; i++) {
            newCounters[i] = nextCounter(recipients[i]);
        }
        emit Deliver(traceParent, recipients, newCounters, clock, messageId, batch, timestampOfSigningKey);
    }
}

